"""
CurrentControlledVoltageSource.py
"""

# Copyright (c) 2018 Teledyne LeCroy, Inc.
# All rights reserved worldwide.
#
# This file is part of SignalIntegrity.
#
# SignalIntegrity is free software: You can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation, either
# version 3 of the License, or any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <https://www.gnu.org/licenses/>

def CurrentControlledVoltageSource(G,Z0=50.):
    """ CurrentControlledVoltageSource
    Current controlled voltage source
    @param G float or complex transresistance.
    @param Z0 (optional) float or complex reference impedance Z0 (defaults to 50 Ohms).
    @return s-parameter matrix of a current controlled voltage source.
    The four port current controlled voltage source has
    the current sense element sensing the current
    entering port 1 and exiting port 2.

    The voltage generated by the amplifier has its positive terminal
    at port 4 and its negative terminal at port 3.
    """
    return  [[0.,1.,0.,0.],
            [1.,0.,0.,0.],
            [-G/(2.*Z0),G/(2.*Z0),0.,1.],
            [G/(2.*Z0),-G/(2.*Z0),1.,0.]]